| package |
package := Package name: 'Cypher'.
package paxVersion: 0;
	basicComment: 'This package provides a simple interface to use encription/decription in two well known algoriths: PC1 and the more modern Blowfish.
The pc1.dll and blowfish.dll can be in the same directory as the image to make them work.

Use Cipher class to send a string to be encripted/decripted with some keywordString

Examples:

string := Cipher blowfishEncript: ''Hello World'' usingKeword: ''Earth'' 
Cipher blowfishDecript: string usingKeword: ''Earth'' 

string := Cipher pc1Encript: ''Hello World'' usingKeword: ''Earth'' 
Cipher pc1Decript: string usingKeword: ''Earth'' 

You can also take a look at the CipherTest to see how to use them.

Note: this piece of software is provided *as is* without any warranty or support. Use at your own risk. It could be freely distributed in any comercial or non-comercial applications with peaceful, ''non-beligerous'', ''pro-human kind'' meanings.

Bugs and coments could be notified to:

Sebastin Sastre
info@seaswork.com.ar
www.seaswork.com.ar
'.


package classNames
	add: #BlowfishLibrary;
	add: #Cipher;
	add: #CipherTest;
	add: #PC1Library;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package allResourceNames: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Camp Smalltalk\SUnit\SUnit';
	yourself).

package!

"Class Definitions"!

Object subclass: #Cipher
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ExternalLibrary subclass: #BlowfishLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
ExternalLibrary subclass: #PC1Library
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TestCase subclass: #CipherTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

"End of package definition"!

"Source Globals"!

"Classes"!

Cipher guid: (GUID fromString: '{FC5F1233-09B1-4884-A18C-DE9412E00FF0}')!
Cipher comment: ''!
!Cipher categoriesForClass!Kernel-Objects! !
!Cipher class methodsFor!

blowfishDecript: aString usingKeword: aKewordString

	"Cipher aString using aKeworkString to make the encription process.
	Note: take care with the nulls, perhaps you should not expect them to be at the end
	of the bufferOut."

	| bufferOut result |

	bufferOut := String new: aString size *2 withAll:32 asCharacter.

	result := BlowfishLibrary default 
		decrypt: aKewordString 
		nKeySize: aKewordString size 
		pBufferIn:  aString 
		nBufferSizeIn: aString size
		pBufferOut: bufferOut 
		nBufferSizeOut: bufferOut size.

	^ bufferOut copyFrom:1 to: result
!

blowfishEncript: aString usingKeword: aKewordString

	"Cipher aString using aKeworkString to make the encription process."

	| bufferOut result |

	bufferOut := String new: aString size *2 withAll:32 asCharacter.

	result := BlowfishLibrary default 
		encrypt: aKewordString 
		nKeySize: aKewordString size 
		pBufferIn:  aString 
		nBufferSizeIn: aString size
		pBufferOut: bufferOut 
		nBufferSizeOut: bufferOut size.

	^ bufferOut copyFrom:1 to: result!

pc1Decript: aString usingKeword: aKewordString

	"Cipher aString using aKeworkString to make the encription process.
	Note: take care with the nulls, perhaps you should not expect them to be at the end
	of the bufferOut."

	| bufferOut result |

	bufferOut := String new: aString size *2 withAll:32 asCharacter.

	result := PC1Library default 
		decrypt: aKewordString 
		nKeySize: aKewordString size 
		pBufferIn:  aString 
		nBufferSizeIn: aString size
		pBufferOut: bufferOut 
		nBufferSizeOut: bufferOut size.

	^ bufferOut copyFrom:1 to: result
!

pc1Encript: aString usingKeword: aKewordString

	"Cipher aString using aKeworkString to make the encription process."

	| bufferOut result |

	bufferOut := String new: aString size *2 withAll:32 asCharacter.

	result := PC1Library default 
		encrypt: aKewordString 
		nKeySize: aKewordString size 
		pBufferIn:  aString 
		nBufferSizeIn: aString size
		pBufferOut: bufferOut 
		nBufferSizeOut: bufferOut size.

	^ bufferOut copyFrom:1 to: result
! !
!Cipher class categoriesFor: #blowfishDecript:usingKeword:!public! !
!Cipher class categoriesFor: #blowfishEncript:usingKeword:!public! !
!Cipher class categoriesFor: #pc1Decript:usingKeword:!public! !
!Cipher class categoriesFor: #pc1Encript:usingKeword:!public! !

BlowfishLibrary guid: (GUID fromString: '{DAEF9752-4A62-462A-9E91-383196E8365E}')!
BlowfishLibrary comment: ''!
!BlowfishLibrary categoriesForClass!External-Libraries! !
!BlowfishLibrary methodsFor!

decrypt: aKey nKeySize: keySize pBufferIn: bufferIn nBufferSizeIn: bufferSizeIn pBufferOut: bufferOut nBufferSizeOut: bufferSizeOut

	"
	int Blowfish_Decrypt(unsigned char *pKey, long nKeySize, unsigned char *pBufferIn, long nBufferSizeIn, unsigned char *pBufferOut, long nBufferSizeOut)"

	<stdcall: hresult Blowfish_Decrypt char* dword char* dword char* dword>
	^self invalidCall!

encrypt: aKey nKeySize: keySize pBufferIn: bufferIn nBufferSizeIn: bufferSizeIn pBufferOut: bufferOut nBufferSizeOut: bufferSizeOut

	"
	int Blowfish_Encrypt(unsigned char *pKey, long nKeySize, unsigned char *pBufferIn, long nBufferSizeIn, unsigned char *pBufferOut, long nBufferSizeOut)"

	<stdcall: hresult Blowfish_Encrypt char* dword char* dword char* dword>
	^self invalidCall! !
!BlowfishLibrary categoriesFor: #decrypt:nKeySize:pBufferIn:nBufferSizeIn:pBufferOut:nBufferSizeOut:!public! !
!BlowfishLibrary categoriesFor: #encrypt:nKeySize:pBufferIn:nBufferSizeIn:pBufferOut:nBufferSizeOut:!public! !

!BlowfishLibrary class methodsFor!

fileName
	"Answer the host system file name of the external library which
	the receiver represents."

	^'Blowfish'! !
!BlowfishLibrary class categoriesFor: #fileName!constants!public! !

PC1Library guid: (GUID fromString: '{7EBC907C-9933-4AEF-96DF-6D2CFD85CA95}')!
PC1Library comment: ''!
!PC1Library categoriesForClass!External-Libraries! !
!PC1Library methodsFor!

decrypt: aKey nKeySize: keySize pBufferIn: bufferIn nBufferSizeIn: bufferSizeIn pBufferOut: bufferOut nBufferSizeOut: bufferSizeOut

	"
	int PC1_Decrypt(unsigned char *pKey, long nKeySize, unsigned char *pBufferIn, long nBufferSizeIn, unsigned char *pBufferOut, long nBufferSizeOut)"

	<stdcall: hresult PC1_Decrypt char* dword char* dword char* dword>
	^self invalidCall!

encrypt: aKey nKeySize: keySize pBufferIn: bufferIn nBufferSizeIn: bufferSizeIn pBufferOut: bufferOut nBufferSizeOut: bufferSizeOut

	"
	int PC1_Encrypt(unsigned char *pKey, long nKeySize, unsigned char *pBufferIn, long nBufferSizeIn, unsigned char *pBufferOut, long nBufferSizeOut)"

	<stdcall: hresult PC1_Encrypt char* dword char* dword char* dword>
	^self invalidCall! !
!PC1Library categoriesFor: #decrypt:nKeySize:pBufferIn:nBufferSizeIn:pBufferOut:nBufferSizeOut:!public! !
!PC1Library categoriesFor: #encrypt:nKeySize:pBufferIn:nBufferSizeIn:pBufferOut:nBufferSizeOut:!public! !

!PC1Library class methodsFor!

fileName
	"Answer the host system file name of the external library which
	the receiver represents."

	^'pc1'! !
!PC1Library class categoriesFor: #fileName!constants!public! !

CipherTest guid: (GUID fromString: '{C22933B8-1768-498C-9185-E1A3E5537BDB}')!
CipherTest comment: ''!
!CipherTest categoriesForClass!Unclassified! !
!CipherTest methodsFor!

testBlowfishLibrary

	| bytes encription fileStream decription |

	fileStream := FileStream read:'blowfish.dll' text:false.
	[ bytes := fileStream contents asString ] ensure:[ fileStream close ].

	encription := Cipher blowfishEncript: bytes usingKeword: 'some clever keyword'.
	decription := Cipher blowfishDecript: encription asString usingKeword: 'some clever keyword'.

	self assert: ( (bytes ~= encription) and:[decription = bytes] ).!

testPC1Library

	| bytes encription decription fileStream |

	fileStream := FileStream read:'pc1.dll' text:false.
	[ bytes := fileStream contents asString] ensure:[ fileStream close ].

	encription := Cipher pc1Encript: bytes usingKeword: 'some clever keyword'.
	decription := Cipher pc1Decript: encription usingKeword: 'some clever keyword'.

	self assert: ( (bytes ~= encription) and:[ bytes = decription ] ).! !
!CipherTest categoriesFor: #testBlowfishLibrary!public! !
!CipherTest categoriesFor: #testPC1Library!public! !

"Binary Globals"!

"Resources"!

